@@ -1,15 +1,15 @@
+use super::args::Args;
 use super::connect::DEFAULT_DB;
-use super::StaticString;
 use crate::cmd::{Durability, ReadMode};
-use crate::proto::Payload;
-use crate::{err, Connection, Query, Result, Session};
+use crate::proto::{Payload, Query};
+use crate::{err, r, Command, Connection, Result, Session};
 use async_stream::try_stream;
 use futures::io::{AsyncReadExt, AsyncWriteExt};
 use futures::stream::{Stream, StreamExt};
 use log::trace;
 use ql2::query::QueryType;
 use ql2::response::{ErrorType, ResponseType};
-use ql2::Frame;
+use reql_macros::CommandOptions;
 use serde::de::DeserializeOwned;
 use serde::{Deserialize, Serialize};
 use serde_json::Value;
@@ -26,9 +26,9 @@ pub(crate) struct Response {
     t: i32,
     e: Option<i32>,
     pub(crate) r: Value,
-    b: Option<Vec<Frame>>,
+    b: Option<Value>,
     p: Option<Value>,
-    n: Option<Vec<i32>>,
+    n: Option<Value>,
 }
 
 impl Response {
@@ -44,7 +44,9 @@ impl Response {
     }
 }
 
-#[derive(Debug, Clone, Serialize, Default, Eq, PartialEq, Ord, PartialOrd, Hash)]
+#[derive(
+    Debug, Clone, CommandOptions, Serialize, Default, Eq, PartialEq, Ord, PartialOrd, Hash,
+)]
 #[non_exhaustive]
 pub struct Options {
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -63,22 +65,6 @@ pub struct Options {
     pub db: Option<Db>,
 }
 
-impl Options {
-    pub fn new() -> Self {
-        Default::default()
-    }
-
-    pub fn noreply(mut self, noreply: bool) -> Self {
-        self.noreply = Some(noreply);
-        self
-    }
-
-    pub fn db<T: StaticString>(mut self, db: T) -> Self {
-        self.db = Some(Db(db.static_string()));
-        self
-    }
-}
-
 #[derive(Debug, Clone, Copy, Serialize, Eq, PartialEq, Ord, PartialOrd, Hash)]
 #[non_exhaustive]
 #[serde(rename_all = "lowercase")]
@@ -117,17 +103,18 @@ impl Arg for Connection {
     }
 }
 
-impl Arg for (&Session, Options) {
+impl Arg for Args<(&Session, Options)> {
     fn into_run_opts(self) -> Result<(Connection, Options)> {
-        let (session, options) = self;
+        let Args((session, options)) = self;
         let conn = session.connection()?;
         Ok((conn, options))
     }
 }
 
-impl Arg for (Connection, Options) {
+impl Arg for Args<(Connection, Options)> {
     fn into_run_opts(self) -> Result<(Connection, Options)> {
-        Ok(self)
+        let Args(arg) = self;
+        Ok(arg)
     }
 }
 
@@ -137,15 +124,15 @@ impl Arg for &mut Session {
     }
 }
 
-impl Arg for (&mut Session, Options) {
+impl Arg for Args<(&mut Session, Options)> {
     fn into_run_opts(self) -> Result<(Connection, Options)> {
-        let (session, options) = self;
+        let Args((session, options)) = self;
         let conn = session.connection()?;
-        (conn, options).into_run_opts()
+        r.args((conn, options)).into_run_opts()
     }
 }
 
-pub(crate) fn new<A, T>(query: Query, arg: A) -> impl Stream<Item = Result<T>>
+pub(crate) fn new<A, T>(query: Command, arg: A) -> impl Stream<Item = Result<T>>
 where
     A: Arg,
     T: Unpin + DeserializeOwned,
@@ -158,7 +145,7 @@ where
             conn.session.inner.mark_change_feed();
         }
         let noreply = opts.noreply.unwrap_or_default();
-        let mut payload = Payload(QueryType::Start, Some(query), opts);
+        let mut payload = Payload(QueryType::Start, Some(Query(&query)), opts);
         loop {
             let (response_type, resp) = conn.request(&payload, noreply).await?;
             trace!("yielding response; token: {}", conn.token);
@@ -196,7 +183,7 @@ where
     }
 }
 
-impl Payload {
+impl Payload<'_> {
     fn encode(&self, token: u64) -> Result<Vec<u8>> {
         let bytes = self.to_bytes()?;
         let data_len = bytes.len();
@@ -219,9 +206,9 @@ impl Connection {
         }
     }
 
-    pub(crate) async fn request(
+    pub(crate) async fn request<'a>(
         &mut self,
-        query: &Payload,
+        query: &'a Payload<'a>,
         noreply: bool,
     ) -> Result<(ResponseType, Response)> {
         self.submit(query, noreply).await;
@@ -231,15 +218,15 @@ impl Connection {
         }
     }
 
-    async fn submit(&self, query: &Payload, noreply: bool) {
+    async fn submit<'a>(&self, query: &'a Payload<'a>, noreply: bool) {
         let mut db_token = self.token;
         let result = self.exec(query, noreply, &mut db_token).await;
         self.send_response(db_token, result);
     }
 
-    async fn exec(
+    async fn exec<'a>(
         &self,
-        query: &Payload,
+        query: &'a Payload<'a>,
         noreply: bool,
         db_token: &mut u64,
     ) -> Result<(ResponseType, Response)> {
