@@ -1,7 +1,8 @@
+use std::sync::Arc;
 use std::{error, fmt, io};
 
 /// The most generic error message in ReQL
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub enum Error {
     Compile(String),
     Runtime(Runtime),
@@ -24,7 +25,7 @@ impl fmt::Display for Error {
 ///
 /// All errors on the server unrelated to compilation. Programs may use this to catch any runtime
 /// error, but the server will always return a more specific error class.
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub enum Runtime {
     /// The query contains a logical impossibility, such as adding a number to a string.
     QueryLogic(String),
@@ -61,7 +62,7 @@ impl fmt::Display for Runtime {
 /// The parent class of `OpFailedError` and `OpIndeterminateError`. Programs may use this
 /// to catch any availability error, but the server will always return one of this class’s
 /// children.
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub enum Availability {
     OpFailed(String),
     OpIndeterminate(String),
@@ -86,14 +87,14 @@ impl fmt::Display for Availability {
 ///
 /// This may be a driver bug, or it may be an unfulfillable command, such as an unserializable
 /// query.
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 #[non_exhaustive]
 pub enum Driver {
     Auth(String),
     ConnectionBroken,
     ConnectionLocked,
-    Io(io::Error),
-    Json(serde_json::Error),
+    Io(io::ErrorKind, String),
+    Json(Arc<serde_json::Error>),
     Other(String),
 }
 
@@ -112,7 +113,7 @@ impl fmt::Display for Driver {
                 f,
                 "another query is running a changefeed on this connection"
             ),
-            Self::Io(error) => write!(f, "{}", error),
+            Self::Io(_, error) => write!(f, "{}", error),
             Self::Json(error) => write!(f, "{}", error),
             Self::Other(msg) => write!(f, "{}", msg),
         }
@@ -121,12 +122,12 @@ impl fmt::Display for Driver {
 
 impl From<io::Error> for Error {
     fn from(err: io::Error) -> Error {
-        Driver::Io(err).into()
+        Driver::Io(err.kind(), err.to_string()).into()
     }
 }
 
 impl From<serde_json::Error> for Error {
     fn from(err: serde_json::Error) -> Error {
-        Driver::Json(err).into()
+        Driver::Json(Arc::new(err)).into()
     }
 }
